!----------------------------------------------------------------------------------------------------------------------------------!
SUBROUTINE cbem_lap_solv
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!********************************  COLLOCATION BEM for 3D LAPLACE EQUATION: SINGULAR FORMULATION  *********************************!
!***************************************  3-NODED TRIANGULAR ELEMENT: ANALYTIC INTEGRATION  ***************************************!
!***********************************************  PIECEWISE CONSTANT APPROXIMATION  ***********************************************!
!***************************************************  Solve the Discretized BIE  **************************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011


USE KINDS
USE SRF_GEOM,  ONLY: nbelt   ! nbelt == Total number of boundary elements
USE SRF_COND,  ONLY: eltbc   ! eltbc(k) = {0,1} ==> {Dirichlet,Neunamm} boundary condition is prescribed on element k
USE DISCR_BIE, ONLY: g,rhs   ! g == Discretized BIE matrix or Left-hand side of the BEM equation
                             ! rhs == Right-hand side of the Discretized BIE
USE POTFLUX,   ONLY: phi,flx ! phi == Vector containing boundary potentials
                             ! flx == Vector containing boundary fluxes


IMPLICIT NONE

 integer(kind=k9) :: k,info,alloc_st
 integer(kind=k9), allocatable :: ipiv(:)



    write(*,100)'           >>>  LU Decomposition of the BIE matrix <<<'    

    allocate(ipiv(nbelt),STAT=alloc_st)
    if ( alloc_st == 0 ) then ! If ipiv is successfully allocated
       call dgetrf(nbelt,nbelt,g,nbelt,ipiv,info) ! LU Decomposition
    else
       write(*,*)'cbem_lap_solv: Allocate array "ipiv" failed with Status=',alloc_st
       stop
    endif

    write(*,100)'           >>>  Back-substitution  <<<'          

    call dgetrs('N',nbelt,1,g,nbelt,ipiv,rhs,nbelt,info) ! Back-substitution
    deallocate(ipiv); deallocate(g)

    write(*,*) 'err ==', info

    loop1: do k=1,nbelt
          if (eltbc(k) == 0) then ! eltbc(k) = 0 ==> Dirichlet data is prescribed on element k
             flx(k) = rhs(k)
          else ! eltbc(k) = 1 ==> Neumann data is prescribed on element k
             phi(k) = rhs(k)
          endif
    enddo loop1
	
   deallocate(rhs,eltbc)


  100 format (1x, a)

!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE cbem_lap_solv
!----------------------------------------------------------------------------------------------------------------------------------!
