!----------------------------------------------------------------------------------------------------------------------------------!
SUBROUTINE cbem_poi_assem
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!********************************  COLLOCATION BEM for 3D POISSON EQUATION: SINGULAR FORMULATION  *********************************!
!************************************  3-NODED TRIANGULAR ELEMENT: SEMI-ANALYTIC INTEGRATION  *************************************!
!***********************************************  PIECEWISE CONSTANT APPROXIMATION  ***********************************************!
!************************  Assemble h and g influence matrices and the contributions of the source term ***************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata
!*** Version:   V.1.1
!*** Released:  Aug 21, 2012

 
USE KINDS
USE SRF_GEOM, ONLY: nbelt ! nbelt == Total number of boundary elements
                          ! For Piecewise constant approximation, nbelt is also equal to the number of collocation points on the surface
USE DISCR_BIE,ONLY: h,g   ! h == Coefficient matrix acting on the vector of boundary potentials <==> potential influence matrix
                          ! g == Coefficient matrix acting on the vector of boundary fluxes <==> flux influence matrix
USE DISCR_BIE,ONLY: srct  ! srct(k) == Contribution of the source term at the k-th collocation point


IMPLICIT NONE

 integer(kind=k9) :: k,i,alloc_st



    allocate(h(nbelt,nbelt), g(nbelt,nbelt), srct(nbelt), STAT=alloc_st)
    if ( alloc_st == 0 ) then ! If h,g,srct are successfully allocated
       loop0a: do k=1,nbelt
	  srct(k) = 0.0_dp
	  loop0b: do i=1, nbelt
             h(i,k) = 0.0_dp; g(i,k) = 0.0_dp
          enddo loop0b
       enddo loop0a

!*** Assemble h and g influence matrices
       loop1a: do k=1,nbelt ! Loop over all boundary elements
	  call cbem_poi_ex_int_ud(k) ! For every triangle k, compute and add the influence contribution to all collocation
	                             ! points in h and g and the contributions of the source term in srct 
       enddo loop1a
    else
       write(*,*)'cbem_poi_assem: Allocate arrays "h,g,srct" failed with Status=',alloc_st
       stop
    endif

!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE cbem_poi_assem
!----------------------------------------------------------------------------------------------------------------------------------!
