
!**********************************************************************************************************************************!
!****************************************************  Useful routines for BEM  ***************************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata
!*** Version:   V.1.1
!*** Released:  Aug 21, 2012


!----------------------------------------------------------------------------------------------------------------------------------! 
FUNCTION gammai(p1,p2,qet,cr1,cr2,zn,zd)
!----------------------------------------------------------------------------------------------------------------------------------!	

!===purpose===! 

!**********************************************************************************************************************************!
!***************************************************  Determine gamma_i  **********************************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011


USE KINDS
USE PARAMS, ONLY: pix2 ! pix2 = 2*pi, pi = 3.1415926535897932


IMPLICIT NONE

 real   (kind=dp), intent(in) :: p1,p2,qet,cr1,cr2,zn,zd
 real   (kind=dp)             :: gammai



	     if ( (p1<0.0_dp) .and. (p2>0.0_dp) ) then ! 1-----x----------2 <==> if Projection of collocation point falls in side k

		         if ( qet<0.0_dp ) then

	                      if ( (cr1>0.0_dp) .and. (cr2<0.0_dp) .and. (zn>0.0_dp) .and. (zd<0.0_dp) ) then
		                gammai = datan2(zn,zd) - pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2>0.0_dp) .and. (zn>0.0_dp) .and. (zd<0.0_dp) ) then
		                gammai = datan2(zn,zd) - pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2<0.0_dp) .and. (zn>0.0_dp) ) then
		                gammai = datan2(zn,zd) - pix2
		              else
		                gammai = datan2(zn,zd)
		              endif
		      
		         elseif ( qet>0.0_dp ) then

	                      if ( (cr1>0.0_dp) .and. (cr2<0.0_dp) .and. (zn<0.0_dp) .and. (zd<0.0_dp) ) then
		                 gammai = datan2(zn,zd) + pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2>0.0_dp) .and. (zn<0.0_dp) .and. (zd<0.0_dp) ) then
		                 gammai = datan2(zn,zd) + pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2<0.0_dp) .and. (zn<0.0_dp) ) then
		                 gammai = datan2(zn,zd) + pix2
		              else
		                gammai = datan2(zn,zd)
		              endif
		      
		         else
		            gammai = 0.0_dp    ! gammai = datan2(zn,zd) = 0.0
		         endif  
		
	     
	     else !  x 1----------------2 or 1----------------2 x <==> if Projection of collocation point does not fall in side k
	       gammai = datan2(zn,zd) 
	     endif

!----------------------------------------------------------------------------------------------------------------------------------! 
END FUNCTION gammai
!----------------------------------------------------------------------------------------------------------------------------------!	


!----------------------------------------------------------------------------------------------------------------------------------! 
FUNCTION pt_tria_tst(w,u,v,theta,theta_id)
!----------------------------------------------------------------------------------------------------------------------------------!	

!===purpose===! 

!**********************************************************************************************************************************!
!*************************************************  Point in Triangle Test  *******************************************************!
!***********************  Given a point with barycentric coordinates (u,v) on a plane containing a triangle  **********************!
!************  Determine whether the point (u,v) is outside, or inside, or on an edge, or at a vertex of the triangle  ************!
!**************************  Set the flag theta_id characterizing the position of the query point (u,v)  **************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011


!*** Triangle with vertices {y1,y2,y3}

!           y3
!	    *
!	   / \
!	  /   \
!	 /     \
!       /       \
!      /	 \
!     /		  \
!    *-------------*
!  y1               y2

! Oriented edge L1 = [y1,y2] <== Edge1
! Oriented edge L2 = [y2,y3] <== Edge2
! Oriented edge L3 = [y3,y1] <== Edge3


USE KINDS
USE PARAMS, ONLY: pi,pix2 ! pi = 3.1415926535897932, pix2 = 2*pi


IMPLICIT NONE

 integer(kind=k9), intent(out) :: theta_id
 real   (kind=dp), intent(in ) :: w,u,v
 real   (kind=dp), intent(in ) :: theta(3)
 real   (kind=dp)              :: pt_tria_tst
!*** Note: (u,v) == Barycentric coordinates of the query point on the plane containing the triangle
!*** Note: w = 1-(u+v)



     theta_id = -1

     if ( u<0.0_dp .or. v<0.0_dp .or. w<0.0_dp ) then
	 theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
     elseif ( u==0.0_dp ) then
           if ( v==0.0_dp ) then
              theta_id = 4; pt_tria_tst = theta(1) ! Query point is at Vertex1
	   elseif ( v==1.0_dp ) then
              theta_id = 6; pt_tria_tst = theta(3) ! Query point is at Vertex3
	   elseif ( v<0.0_dp .or. v>1.0_dp ) then
	      theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
	   else ! v>0.0_dp & v<1.0_dp
	      theta_id = 3; pt_tria_tst = pi ! Query point is on Edge3 of the triangle
	   endif
     elseif ( v==0.0_dp ) then
	   if ( u==1.0_dp ) then
              theta_id = 5; pt_tria_tst = theta(2) ! Query point is at Vertex2
	   elseif ( u<0.0_dp .or. u>1.0_dp ) then
	      theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
	   else ! u>0.0_dp & u<1.0_dp
	      theta_id = 1; pt_tria_tst = pi ! Query point is on Edge1 of the triangle
	   endif
     elseif ( w==0.0_dp ) then
	   if ( u<0.0_dp .or. v<0.0_dp ) then
	      theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
	   else ! u>0.0_dp & v>0.0_dp
	      theta_id = 2; pt_tria_tst = pi ! Query point is on Edge2 of the triangle
	   endif   
     else ! u>0.0_dp & v>0.0_dp & w>0.0_dp
	theta_id = 0; pt_tria_tst = pix2 ! Query point is Inside the triangle
     endif


!----------------------------------------------------------------------------------------------------------------------------------! 
END FUNCTION pt_tria_tst
!----------------------------------------------------------------------------------------------------------------------------------!	


!----------------------------------------------------------------------------------------------------------------------------------!
FUNCTION asym_chi(p1,p2,d)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!************************************  return ASYMTOTIC VALUE of chi when q^2 + et^2 = d << 1  ************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011

 
USE KINDS


IMPLICIT NONE

 real(kind=dp), intent(in) :: p1,p2,d
 real(kind=dp)             :: asym_chi

 real(kind=dp)             :: z1,z2
 
 

     if ( dabs(p1)>0.0_dp .and. dabs(p2)>0.0_dp ) then
           z1 = (d/p1)/p1; z2 = (d/p2)/p2
           asym_chi = dlog( dabs(p2/p1)*( (1.0_dp-0.25_dp*z1*(1.0_dp-0.5_dp*z1))/(1.0_dp-0.25_dp*z2*(1.0_dp-0.5_dp*z2)) ) )
     else
           asym_chi = 0.0_dp
     endif

!----------------------------------------------------------------------------------------------------------------------------------!
END FUNCTION asym_chi
!----------------------------------------------------------------------------------------------------------------------------------!


!----------------------------------------------------------------------------------------------------------------------------------!
SUBROUTINE ortho_comp_basis(t1,t2,e1,e2,e3)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!************************************  Given two linearly independent vectors t1 and t2 in 3D  ************************************!
!** Compute the orthonormal triad {e1,e2,e3} such that e1=t1/norm(t1), norm(e2) = 1 and e2 is orthogonal to e1, and e3 = e1 x e2 **!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011

 
USE KINDS


IMPLICIT NONE

 real   (kind=dp), intent(in ) :: t1(3),t2(3)
 real   (kind=dp), intent(out) :: e1(3),e2(3),e3(3)

 integer(kind=k9)              :: i
 real   (kind=dp)              :: al,snrm_te,nrm_te,nrm_tx



!*********  Orthonormal local basis: e1(3),e2(3),e3(3)  ***************************************************************************!
    snrm_te = t1(1)*t1(1) + t1(2)*t1(2) + t1(3)*t1(3)
    nrm_te = dsqrt(snrm_te)

    al = (t1(1)*t2(1) + t1(2)*t2(2) + t1(3)*t2(3))/snrm_te
    
    loop1: do i=1,3 ! Loop over all components
       e2(i) = t2(i) - al*t1(i)
    enddo loop1

    nrm_tx = dsqrt(e2(1)*e2(1) + e2(2)*e2(2) + e2(3)*e2(3))

    loop2: do i=1,3 ! Loop over all components
       e1(i) = t1(i)/nrm_te; e2(i) = e2(i)/nrm_tx
    enddo loop2

!*** Unit Normal vector to the plane spanned by {e1,e2}: e3 = e1 x e2
    e3(1) = e1(2)*e2(3) - e2(2)*e1(3)
    e3(2) = e1(3)*e2(1) - e2(3)*e1(1)
    e3(3) = e1(1)*e2(2) - e2(1)*e1(2)

    return

!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE ortho_comp_basis
!----------------------------------------------------------------------------------------------------------------------------------!


!----------------------------------------------------------------------------------------------------------------------------------!
FUNCTION intsimp(x1,x2,f,nmax,eps)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!******** Compute the 1D integral of a real-valued function f from x1 to x2 using successive refinements of Simpson's rule ********!
!************* The 1D integral $\int\limits_{x1}^{x2} f(x) dx$ is evaluated until a specified accuracy eps is achieved ************!
!************************************* or until a maximum number of iterations nmax is reached ************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.0
!*** Released:  May 28, 2011

 
! Parameter list:
! x1,x2 == respectively lower and upper limits of the integral $\int\limits_{x1}^{x2} f(x) dx$
! f     == integrand
! nmax  == Maximum number of Simpson's rule iterations allowed
! eps   == Absolute tolerance on Simpson's rule


USE KINDS


IMPLICIT NONE

! Parameters:
 integer(kind=k9), intent(in)  :: nmax
 real   (kind=dp), intent(in)  :: x1,x2
 real   (kind=dp), intent(in)  :: eps
 real(kind=dp) :: intsimp

!----------------------------------------------------------------------------------------------------------------------------------!
INTERFACE
!----------------------------------------------------------------------------------------------------------------------------------!

!----------------------------------------------------------------------------------------------------------------------------------!
  FUNCTION f(x)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!*************************************** Return the value of the function f at x, i.e., f(x) **************************************!
!**********************************************************************************************************************************!

   USE KINDS

   IMPLICIT NONE
   
    real(kind=dp), intent(in) :: x
    real(kind=dp) :: f

!----------------------------------------------------------------------------------------------------------------------------------!    
  END FUNCTION f
!----------------------------------------------------------------------------------------------------------------------------------!  

!----------------------------------------------------------------------------------------------------------------------------------!
END INTERFACE
!----------------------------------------------------------------------------------------------------------------------------------!

! Local variables:
 integer(kind=k9) :: i,n,m
 real   (kind=dp) :: trapzd0,trapzd  ! Trapezoidal rule
 real   (kind=dp) :: simp0,simp ! Simpson's rule
 real   (kind=dp) :: x,delta,sumf,toler



    delta = x2-x1
    trapzd0 = 0.5_dp*(f(x1)+f(x2))*delta ! trapzd == Trapezoidal rule at the iteration 0, i.e. when n=0
    delta = 0.5_dp*delta
    x = x1 + delta ! x == midpoint of the interval [x1,x2]
    trapzd = 0.5_dp*trapzd0 + delta*f(x) ! trapzd == Composite trapezoidal rule at the first iteration, i.e. when n=1
    simp0 = (4.0_dp*trapzd - trapzd0)/3.0_dp
    simp = simp0 ! simp == Composite Simpson's rule at the first iteration, i.e. when n=1

    m = 1; trapzd0 = trapzd; toler = 15.0_dp*eps
    loop0a: do n=2,nmax ! Loop over the remaining iterations
       m = m*2 ! m = 2**(n-1)
       delta = 0.5_dp*delta ! delta == Step-size at iteration n
       sumf = 0.0_dp
       loop0b: do i=1,m ! Loop over the additional integration points
          x = x1 + real(2*i-1,dp)*delta
	  sumf = sumf + f(x)
       enddo loop0b
       trapzd = 0.5_dp*trapzd0 + delta*sumf  ! trapzd == Composite trapezoidal rule at iteration n
       simp = (4.0_dp*trapzd - trapzd0)/3.0_dp ! simp == Composite Simpson's rule at iteration n
       if (dabs(simp-simp0)<toler) exit loop0a
       trapzd0 = trapzd; simp0 = simp
    enddo loop0a  ! End Loop over the remaining iterations
    
    if (nmax<2) write(*,*)'Warning: only 1 iteration of Simpson''s rule in function intsimp, integral may not be accurate'
    if (n-1==nmax) write(*,*)"Warning:",nmax, &
    "iterations of Simpson's rules has been reached in function intsimp, integral may not be accurate"

    intsimp = simp
    
!----------------------------------------------------------------------------------------------------------------------------------!
END FUNCTION intsimp
!----------------------------------------------------------------------------------------------------------------------------------!


!----------------------------------------------------------------------------------------------------------------------------------!
FUNCTION strl_int(x,y)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!****************************  Given a source point x, a field point y, and a source function src_func  ***************************!
!*****************************  Compute the straight-line integral \int_0^1 t src_func(x + t(y-x)) dt  ****************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.0
!*** Released:  May 28, 2011


USE KINDS
USE PARAMS, ONLY: nmax_simp ! nmax_simp == Maximum number of Simpson's rule iterations for the integration of the Newton potential
USE PARAMS, ONLY: simp_eps  ! simp_eps == Absolute tolerance on Simpson's rule for the integration of the Newton potential
USE GLB_VAR_TMP, ONLY: src_pt,dir_vec ! src_pt == Source point in 3D space; dir_vec == direction vector of the integration line


IMPLICIT NONE

 real   (kind=dp), intent(in) :: x(3),y(3) ! x == Source point in 3D space; y == field point in 3D space
 real   (kind=dp) :: strl_int

 real   (kind=dp), external :: sfunc ! sfunc(t) == t*src_func(x + t(y-x))
 real   (kind=dp), external :: intsimp ! intsimp == Evaluate the integral \int_0^1 sfunc(t) dt using Simpson's rule



    dir_vec(1) = y(1)-x(1); dir_vec(2) = y(2)-x(2); dir_vec(3) = y(3)-x(3) ! dir_vec = y-x
    src_pt(1) = x(1); src_pt(2) = x(2); src_pt(3) = x(3) ! src_pt = x
    strl_int = intsimp(0.0_dp,1.0_dp,sfunc,nmax_simp,simp_eps)

!----------------------------------------------------------------------------------------------------------------------------------!
END FUNCTION strl_int
!----------------------------------------------------------------------------------------------------------------------------------!


!----------------------------------------------------------------------------------------------------------------------------------!
FUNCTION sfunc(t)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!*********************************** Return the value of the function sfunc at t, i.e., sfunc(t) **********************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.0
!*** Released:  May 28, 2011


USE KINDS
USE GLB_VAR_TMP, ONLY: src_pt,dir_vec ! src_pt == Source point in 3D space; dir_vec == direction vector of the integration line


IMPLICIT NONE

 real(kind=dp), intent(in) :: t
 real(kind=dp) :: sfunc ! sfunc(t) == t*src_func(x + t*dir_vec)

 integer(kind=k9) :: k
 real   (kind=dp) :: z(3)
 real   (kind=dp), external :: src_func ! src_func == Source function



    loop0: do k=1,3 ! Loop over all components
       z(k) = src_pt(k) + t*dir_vec(k)
    enddo loop0
    sfunc = t*src_func(z)

!----------------------------------------------------------------------------------------------------------------------------------!    
END FUNCTION sfunc
!----------------------------------------------------------------------------------------------------------------------------------!  
